/*
 *  SokoBan, a Sokoban game.
 *  Copyright (C) 2008  Colomban Wendling <ban@herbesfolles.org>
 * 
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */


#include <stdlib.h>

#include "sokoban.h"
#include "map.h"
#include "player.h"
#include "i18n.h"
#include "level.h"


/**
 * \defgroup level Level managment functions
 * Level loading, unloading & managment
 */

/** @{ */



/**
 * \brief Load a level from a file
 * \param filename The path of the level to load
 * \return TRUE if no error, FALSE else
 * 
 * \see level_loadFromStream
 * 
 * This function load a level to the defaul map (\see map.c) from the
 * sepecified path. If an error occurs, it will be printed to the stderr ouput
 * and the function will return FALSE.
 */
MyBool level_load (const char* filename)
{
   FILE *fp;
   MyBool rv = FALSE;
   
   fp = fopen (filename, "r");
   
   if (fp != NULL)
   {
      rv = level_loadFromStream (fp);
      fclose (fp);
   }
   else
      perror (_("** Failed to open level"));
   
   return rv;
}

/**
 * \brief Load a level from a file stream
 * \param stream The file stream to read
 * \return TRUE if no error, FALSE else
 * 
 * This function load a level to the defaul map (\see map.c) from the
 * sepecified file stream. If an error occurs, the function will return FALSE.
 */
MyBool level_loadFromStream (FILE *stream)
{
   int i, j;
   char c;
   
   for (i = 0; i < map_getHeight (); i++)
   {
      for (j = 0; j < map_getWidth (); j++)
      {
         switch ((c = fgetc (stream)))
         {
            case 'W':
               map_setBlockAtPos (j, i, WALL);
               break;
            
            case ' ':
               map_setBlockAtPos (j, i, GROUND);
               break;
            
            case 'b':
               map_setBlockAtPos (j, i, BOX);
               break;
            
            case 'B':
               map_setBlockAtPos (j, i, BOXDONE);
               break;
            
            case '.':
               map_setBlockAtPos (j, i, POINT);
               break;
            
            case '\r':
            case '\n':
               /* skip EOL */
               j --;
               break;
            
            case 's': /* player starts on gournd */
               map_setBlockAtPos (j, i, GROUND);
               
               player_setPos (j, i);
               break;
            
            case 'S': /* player starts on point */
               map_setBlockAtPos (j, i, POINT);
               
               player_setPos (j, i);
               break;
            
            case EOF:
               /* check if all the map was filled */
               if (j != map_getWidth () || i != map_getHeight ())
               {
                  fprintf (stderr, _("** Bad level dimetions\n"));
                  return FALSE;
               }
               break;
            
            default:
               fprintf (stderr, _("** Unknown block at %d:%d: %c\n"), i, j, c);
               
               map_setBlockAtPos (j, i, UNKNOWN);
         }
      }
   }
   
   /* reset moves of the player at level load */
   player_resetMoves ();
   
   return TRUE;
}

/**
 * \brief Write a level to a file
 * \param filename the path of the file where write data
 * \param ppx the X player position
 * \param ppy the Y player position
 * \return TRUE if no error, FALSE else
 * 
 * \see level_writeToStream
 * 
 * This function write the default map to the specified file.
 */
MyBool level_write (const char* filename, int ppx, int ppy)
{
   FILE *fp;
   MyBool rv = FALSE;
   
   fp = fopen (filename, "w");
   
   if (fp != NULL)
   {
      rv = level_writeToStream (fp, ppx, ppy);
      fclose (fp);
   }
   else
      perror (_("** Failed to write level:"));
   
   return rv;
}


/**
 * \brief Write a level to a file stream
 * \param fp the file stream where write data
 * \param ppx the X player position
 * \param ppy the Y player position
 * \return TRUE if no error, FALSE else
 * 
 * \see map
 * 
 * This function write the default map to the stream pointed by \p fp
 */
MyBool level_writeToStream (FILE *fp, int ppx, int ppy)
{
   int i, j;
   char c;
   
   for (i = 0; i < map_getHeight (); i++)
   {
      for (j = 0; j < map_getWidth (); j++)
      {
         if (ppx == j && ppy == i)
         {
            if (map_checkBlockAtPos (j, i, POINT))
               c = fputc ('S', fp); /* player start position */
            else
               c = fputc ('s', fp); /* player start position */
         }
         else
         {
            switch (map_getBlockAtPos (j, i))
            {
               case WALL:
                  c = fputc ('W', fp);
                  break;
               
               case GROUND:
                  c = fputc (' ', fp);
                  break;
               
               case BOX:
                  c = fputc ('b', fp);
                  break;
               
               case BOXDONE:
                  c = fputc ('B', fp);
                  break;
               
               case POINT:
                  c = fputc ('.', fp);
                  break;
               
               default:
                  fprintf (stderr, _("** Unknown block at %d:%d\n"), i, j);
                  return FALSE;
            }
         }
         
         if (c == EOF)
            return FALSE;
      }
      
      /* add EOL */
      if (fputs (EOL, fp) == EOF)
         return FALSE;
   }
   
   return TRUE;
}

/** @} */
